//Definim les dependencies mitjançant require.js

define(['jquery', 
    'underscore', 
    'Backbone', 
    'models/ClientReserva',
    'models/Reserva',
    'models/Hotel',
    'models/Promocio',
    'models/ReservaPromocio',
    'views/LlistatPromocions/ReservaPromocioView',
    'text!templates/LlistatPromocions/LlistatPromocionsView.tpl','jqm'],
    function ($, _, Backbone, ClientReserva, Reserva, Hotel, Promocio, ReservaPromocio, ReservaPromocioView, LlistatPromocionsViewTemplate) {
        return Backbone.View.extend({

            $promocionsPerImprimir:null,

            vectorPromocions: [],

            template:_.template(LlistatPromocionsViewTemplate),

            // Events a la pantalla i respectives funcions que els atendran

            events: {
	      		'click #menu': 'popUp',
                'click #llista-promocions li': 'accesPromocio',
                'pageshow':'pageShow'
	    	},


            render:function () {
                // Dibuixam la pantalla mitjançant la plantilla HTML
                this.$el.html(this.template());

                // I després es dibuixen els element que la forman, en aquest cas:
                //  Llistat de promocions
                this.$promocionsPerImprimir = this.$('#llista-promocions');
                return this;
            },

            // Crida al menu emergent de la aplicació
	    	popUp: function(e) {
	      		$("#pop").popup("open");
	    	},

            // Funció que controla la correcte generació del llistat que apareix en pantalla
            pageShow:function (event) {
                
                var that = this;
                _.defer(function () {
                    that.imprimirPromocions();
                });
            },

            // Definició del llistat de promocions actives del hotel
            // generat mitjançant una serie de consultes de la BBDD a parse.com

            imprimirPromocions:function() {

                var that = this;

                $.mobile.showPageLoadingMsg('a', 'Carregant promocions actives del hotel');


                // Crearem el llistat de promocions actives per reservar
                // assenyalant amb un boolea les ja reservades.
                // Finalment imprimirem del promocions actives no reservades

                hotelClient = new Parse.Query(ClientReserva);
                hotelClient.include('reserva.hotel');
                hotelClient.equalTo('usuari', Parse.User.current());

                that.reserva = new Reserva();
                that.hotel = new Hotel();


                hotelClient.first({
                    success:function (usuari) {

                        console.log('El codi del usuari es el ' + Parse.User.current().get('username'));

                        //Seleccionem l'hotel on s'ha enregistrat per saber les seves promocions
                        
                        that.reserva = usuari.get('reserva');
                        that.hotel = that.reserva.get('hotel');

                        console.log('Lhotel  es ' + that.hotel.get('Nom'));



                        promocionsHotelUsuari = new Parse.Query(Promocio);
                        promocionsHotelUsuari.include('hotel');
                        promocionsHotelUsuari.equalTo('hotel', that.hotel);

                        //Omplim el vector amb les promocions disponibles per l'hotel del client

                        promocionsHotelUsuari.find({
                            success:function (llistaPromocions){

                                $.each(llistaPromocions, function (i,promocio){
                                    that.vectorPromocions[i] = promocio;
                                    console.log('Inserim al vector la promocio  ' + promocio.get('Nom'));
                                });

                            },error:function (error) {
                                $.mobile.hidePageLoadingMsg();
                                alert("Error en l'iniciació del vector de promocions");
                            }
                        });



                        // Es consulten les promocions ja reservades per l'usuari i es marquen al vector de promocions
                        promocionsReservadesClient = new Parse.Query(ReservaPromocio);
                        promocionsReservadesClient.include('promocio');
                        promocionsReservadesClient.equalTo('usuari', Parse.User.current());

                        var promocioReservada = new Promocio();

                        promocionsReservadesClient.find({
                            success:function (llistaReserves){

                                $.each(llistaReserves, function (i,reserva){

                                    var i = 0;
                                    promocioReservada = reserva.get('promocio');

                                    while ( promocioReservada.id != that.vectorPromocions[i].id ){
                                        i= i+1;
                                    }

                                    if (promocioReservada.id == that.vectorPromocions[i].id){
                                        console.log('Marcam la promocio ' 
                                            + that.vectorPromocions[i].get('Nom') 
                                            + ' com a reservada al vector');
                                        that.vectorPromocions[i].set('reservada', true);                           
                                    }
                                });



                                // Comprovam finalment si cada promoció llistada ja s'ha reservat
                                // Si s'ha reservat no la imprimim  

                                var items = [];
                                $.mobile.hidePageLoadingMsg();

                                $.each(that.vectorPromocions, function (i,promocio){

                                    console.log('Estudiam si imprimim promocio :' + promocio.get('Nom'));
     
                                   if (!promocio.get('reservada')){
                                        console.log('Imprimir promocio ' + promocio.get('Nom'));

                                        var $item = $('<li id= "'
                                            + i
                                            +'"> <h3>' 
                                            + promocio.escape('Nom')
                                            + '</h3> </li>').jqmData('promocio',promocio);

                                        items.push($item[0]);
                                    }

                                });

                                that.$promocionsPerImprimir.html(items).listview('refresh');


                            },error:function (error) {
                                $.mobile.hidePageLoadingMsg();
                                alert("Error en el llistat de reserves");
                            }
                        });


                    }, error:function (error) {
                        $.mobile.hidePageLoadingMsg();
                        alert("Usuari mal registrat. No existeix client assignat.");
                    }
                });

            },


            accesPromocio:function (event){
                var promo = new Promocio();
                promo = $(event.currentTarget).jqmData('promocio'); 
                console.log('La promocio a parametritzar es ' + promo.get('Nom'));
                $.mobile.jqmNavigator.pushView(new ReservaPromocioView({model:promo}));
            }

        });
    });